unit ob_u;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, jpeg, ExtDlgs, Menus, ComCtrls,
  inifiles,Grids, Spin,Buttons,math;

procedure ob_rdimg(fn:string);
procedure ob_mkpixelbmp(prevok:bool);
procedure ob_mkhistoimg;
procedure ob_entfernen;
function ob_isempty:bool;
procedure ob_imgMouseMove(
  Sender: TObject;
  Shift: TShiftState;
  X,Y: Integer
);

implementation

uses hauptu,op_u;

//load original-image from file------------------------
procedure ob_rdimg(fn:string);
begin
  screen.Cursor:=crhourglass;
  try
    //remove previous
    ob_entfernen;
    if not fileexists(fn) then exit;

    try
      //get new one
      hauptf.ob_img.Picture.loadfromfile(fn);
      hauptf.ob_fne.text:=fn;

      //save original to bmp
      hauptf.ob_bmp.assign(
        hauptf.ob_img.Picture.graphic
      );

      //set hint informations
      hauptf.ob_img.Hint:=
        'Original-Bild "'+hauptf.ob_fne.text+'"'+_cr+
        'Dimension: '+
        inttostr(hauptf.ob_bmp.width)+
        ' x '+
        inttostr(hauptf.ob_bmp.Height)+
        ' Pixel';
    except
    end;
  finally
    //make histogram & preview
    ob_mkhistoimg;
    op_updateprev;
    hauptf.setbuttons;
    screen.Cursor:=crdefault;
  end;
end;

//create pixelbmp: reduce original to quad-dimension
//=> every pixel represents one quader
procedure ob_mkpixelbmp(prevok:bool);
var
  rec:trect;
  w,h:integer;
begin
  //set options-infos
  op_u.op_setebinfo;

  //is there an original-pic
  if ob_isempty then exit;

  //get dimension of quads
  w:=hauptf.op_qhorzse.value;
  h:=strtoint(hauptf.op_qverte.text);

  //just preview?
  if not prevok then begin
    //no: want smooth-mode?
    if hauptf.op_verlaufchb.checked then begin
      //yep, increase dimension
      //(smooth needs 3x3 pixels for one quader)
      w:=w*3;
      h:=h*3;
    end;
  end;

  //set dimension of pixelbmp
  hauptf.ob_pixelbmp.width:=w;
  hauptf.ob_pixelbmp.Height:=h;

  //reduce original to pixelbmp
  if prevok or (w<4) then begin
    //fast stretch
    rec:=rect(0,0,w,h);
    hauptf.ob_pixelbmp.Canvas.StretchDraw(rec,hauptf.ob_bmp);
  end
  else begin
    //optimized thumb
    op_u.op_optthumb(hauptf.ob_bmp,hauptf.ob_pixelbmp);
  end;
end;

//create gray-value-histogram of original
procedure ob_mkhistoimg;
var
  x,y,c,helligkeit:integer;
  col:tcolor;
  ca:array[0..255]of double;
  pba:pbytearray;
begin
  //reset gray-value-array
  for c:=0 to 255 do ca[c]:=0;

  if hauptf.ob_bmp.height>0 then begin
    //fill gray-array with numbers
    //of gray-values in original-bmp
    for y:=1 to hauptf.ob_bmp.height-1 do begin
      pba:=hauptf.ob_bmp.ScanLine[y];
      for x:=1 to hauptf.ob_bmp.width-1 do begin
        //get pixel-color
        col:=hauptf.pba2col(pba,x);

        //convert to gray
        helligkeit:=hauptf.col2helligkeit(col);

        //increase gray-value in array
        ca[helligkeit]:=ca[helligkeit]+1;
      end;
    end;

    //save gray-array
    for c:=0 to 255 do hauptf.ob_histoa[c]:=ca[c];

    //want logarithm norm?
    if hauptf.ob_logchb.checked then begin
      for c:=0 to 255 do ca[c]:=ln(1+ca[c]);
    end;

  end;

  //transfer gray-array to histogram-image
  hauptf.mkhistoimg(hauptf.ob_histoimg,ca);

  //save histogram
  hauptf.ob_histobmp.assign(hauptf.ob_histoimg.Picture);
end;

//remove original-pic-------------------------
procedure ob_entfernen;
begin
  hauptf.ob_img.Picture.Graphic:=nil;
  hauptf.ob_img.Hint:='Kein Quellbild';
  hauptf.ob_fne.Text:='';
  ob_mkhistoimg;
  op_updateprev;
  hauptf.setbuttons;
end;

//check if original-bild is loaded
function ob_isempty:bool;
begin
  result:=(hauptf.ob_img.Picture.Graphic=nil);
end;

//mouse moves on original image----------------------
//set "cursor" in histogram
procedure ob_imgMouseMove(
  Sender: TObject;
  Shift: TShiftState;
  X,Y: Integer
);
var
  l,t,w,h,
  helligkeit,hfg:integer;
  bmp:tbitmap;
  col:tcolor;
  d,hfgproz:double;
  s:string;
begin
  hauptf.ob_sh.brush.color:=clsilver;
  hauptf.ob_helle.Text:='';
  hauptf.ob_haeufe.Text:='';

  if ob_isempty then exit;

  bmp:=tbitmap.create;
  try
    //get backup of histogram
    bmp.assign(hauptf.ob_histobmp);

    //calculate position of "inner" image in paintbox
    hauptf.getinnerbounds(
      hauptf.ob_img.Width,
      hauptf.ob_img.height,
      hauptf.ob_bmp.Width,
      hauptf.ob_bmp.height,
      l,t,w,h
    );

    //mouse over image?
    x:=x-l;if(x<0)or(x>w)then exit;
    y:=y-t;if(y<0)or(y>h)then exit;

    //convert position to original-image
    d:=hauptf.ob_bmp.Width/w;
    x:=trunc(x*d);
    y:=trunc(y*d);

    //get color-value under mouse
    col:=hauptf.ob_bmp.canvas.Pixels[x,y];
    hauptf.ob_sh.Brush.color:=col;

    //convert to gray-value
    helligkeit:=hauptf.col2helligkeit(col);
    hauptf.ob_helle.Text:=inttostr(helligkeit);

    //show number of gray-values in hitogram-array
    hfg:=trunc(hauptf.ob_histoa[helligkeit]);
    s:=inttostr(hfg);
    hfgproz:=(hfg*100)/(hauptf.ob_bmp.Width*hauptf.ob_bmp.height);
    s:=s+' ('+format('%f',[hfgproz])+'%)';
    hauptf.ob_haeufe.Text:=s;

    //paint "cursor"-line in histogram
    x:=helligkeit;
    bmp.canvas.pen.width:=1;
    bmp.canvas.pen.color:=clgreen;
    bmp.Canvas.MoveTo(x,0);
    bmp.Canvas.lineTo(x,bmp.height);

  finally
    //show (new) histogram
    hauptf.ob_histoimg.picture.assign(bmp);
    bmp.free;
  end;
end;

end.

